# -*- coding: utf-8 -*-

import luigi
import law
import os
import re
import shutil

from subprocess import PIPE
from law.util import interruptable_popen

from MergeFinal import MergeFinal

from analysis.framework import Task

class AddStatunc(Task, law.LocalWorkflow):
  my_debug = luigi.BoolParameter(default=False)

  merge_dir = luigi.Parameter(default='grids')
  steer_dir = luigi.Parameter()

  orders      = luigi.ListParameter()
  observables = luigi.ListParameter()
  rivetid     = luigi.Parameter()
  histoids    = luigi.ListParameter()

  def create_branch_map(self):
    i = 0
    branchmap = {}
    for order in self.orders:
      j = 0
      for obs in self.observables:
        branchmap[i] = {
          'iobs': j,
          'obs': obs,
          'order': order
        }
        j += 1
        i += 1
    return branchmap

  def workflow_requires(self):
    return {
      'mergefinal': MergeFinal()
    }

  def requires(self):
    return MergeFinal()

  def output(self):
    return law.LocalFileTarget('{}/{}/Combined/Final/{}.{}.{}_v25.tab.gz'.format(self.merge_dir, self.name, self.process, self.branch_data['order'], self.branch_data['obs']))

  def run(self):
    try:

      opath = os.getcwd()
      tabpath = '{}/{}/Combined/Final'.format(self.merge_dir, self.name)
      tabbase = '{}.{}.{}'.format(self.process, self.branch_data['order'], self.branch_data['obs'])
      tabfile = tabbase + '.tab.gz'
      outfile = tabbase + '_v25.tab.gz'
      strfile = tabbase + '.str'
      strtemplate = '{}/SteerModify.str'.format(self.steer_dir)
      strpathfile = tabpath + '/' + strfile

      # Write per table strfile
      # Add RIVET_IDs if desired; requires SteerModify.str to contain a scenario description with RIVET_ID=
      histoid = ""
      if self.rivetid:
        print('[AddStatunc]: Found RIVET_ID {}'.format(self.rivetid))
      if self.histoids:
        histoid = self.histoids[self.branch_data['iobs']]
        print('[AddStatunc]: Found HISTO_ID {}'.format(histoid))
      RID = 'RIVET_ID='
      with open(strpathfile, 'w') as ofile:

        with open(strtemplate, 'r') as ifile:
          for line in ifile:
            if RID in line and self.rivetid and histoid:
              if self.my_debug:
                print('[AddStatunc]: Changing input line with RIVET_ID\n{}'.format(line))
              line = line.replace(RID, RID+self.rivetid+"/"+histoid)
              if self.my_debug:
                print('[AddStatunc]: to line\n{}'.format(line))
            ofile.write(line)
        # ifile closed automatically leaving 'with' block

        # Append information for adding statistical (numerical) uncertainty
        print('[AddStatunc]: Append modification specification to strfile {}\n'.format(strpathfile))
        ofile.write('# Attaching missing InfoBlock information\n')
        ofile.write('InfoBlockFiles {\n')
        lofile   = re.sub(self.branch_data['order'],'LO',tabbase) + '.dat' + '\n'
        nlofile  = re.sub(self.branch_data['order'],'NLO_only',tabbase) + '.dat' + '\n'
        nnlofile = re.sub(self.branch_data['order'],'NNLO_only',tabbase) + '.dat' + '\n'
        ofile.write('  '+lofile)
        if self.branch_data['order'] != 'LO':
          ofile.write('  '+nlofile)
        if self.branch_data['order'] == 'NNLO':
          ofile.write('  '+nnlofile)
        ofile.write('}\n')
        ofile.write('InfoBlockOrders {\n')
        ofile.write('  LO\n')
        if self.branch_data['order'] != 'LO':
          ofile.write('  NLO\n')
        if self.branch_data['order'] == 'NNLO':
          ofile.write('  NNLO\n')
        ofile.write('}\n')
      # ofile closed automatically leaving 'with' block

      # Change to dir with final grids and dat files
      os.chdir(tabpath)
      if self.my_debug:
        print('[AddStatunc]: Changed path to: {}'.format(os.getcwd()))
        print('[AddStatunc]: Try to open strfile {}'.format(strfile))

      if self.my_debug:
        print('[AddStatunc]: DEBUG: Running command fnlo-tk-modify steerfile={} InTable={} OutTable={}'.format(strfile, tabfile, outfile))
      code, out, error = interruptable_popen(['fnlo-tk-modify', 'steerfile={}'.format(strfile), 'InTable={}'.format(tabfile), 'OutTable={}'.format(outfile)], stdout=PIPE, stderr=PIPE)
      if self.my_debug:
        print('[AddStatunc]: DEBUG: Printing subprocess code:\n{}\n'.format(code))
        print('[AddStatunc]: DEBUG: Printing subprocess output:\n{}\n'.format(out))
        print('[AddStatunc]: DEBUG: Printing subprocess error:\n{}\n'.format(error))
      # Change back to original path
      os.chdir(opath)
      if self.my_debug:
        print('[AddStatunc]: Changed path back to: {}'.format(os.getcwd()))
      if (code != 0):
        raise Exception(error + 'fnlo-tk-modify returned non-zero exit status {}'.format(code))

    except:
      print('[AddStatunc]: ERROR running fnlo-tk-modify, aborted!')
      raise
