# -*- coding: utf-8 -*-

import glob
import law
import luigi
import os
import re
import shutil

from law.util import interruptable_popen
from subprocess import PIPE

from FnloCppreadFinal import FnloCppreadFinal

from analysis.framework import Task

class DataComparison(Task, law.LocalWorkflow):
  my_debug = luigi.BoolParameter(default=False)

  merge_dir = luigi.Parameter(default='grids')
  plots_dir = luigi.Parameter(default='plots')

  orders      = luigi.ListParameter()
  observables = luigi.ListParameter()
  rivetid     = luigi.Parameter()

  pdf         = luigi.Parameter()
  scalenames  = luigi.ListParameter()

  def create_branch_map(self):
    branchmap = {}
    i = 0
    for sclnam in self.scalenames:
      branchmap[i] = {
        'scalename': sclnam
      }
      i += 1
    return branchmap

  def workflow_requires(self):
    return {
      'fnlocppread': FnloCppreadFinal(self)
    }

  def output(self):
    return law.LocalDirectoryTarget('{}/{}/DataComparison/{}'.format(self.plots_dir, self.name, self.branch_data['scalename']))

  def run(self):
    try:
      # Some initial definitions
      opath = os.getcwd()
      tabpath = '{}/{}/Combined/Final'.format(self.merge_dir, self.name)
      tabext  = '_v25.tab.gz'
      tabunc  = '6P'
      yodanames = {}
      yodafiles = {}

      # Change to dir with final grids and dat files
      os.chdir(tabpath)
      if self.my_debug:
        print('[DataComparison]: Changed path to: {}'.format(os.getcwd()))

      # Initial remove of potentially existing yoda files to collect order-wise results for all observables
      for order in self.orders:
        yodanames[order] = '{}.{}.{}.{}.{}.{}.yoda'.format(self.process, self.name, order, self.pdf, tabunc, self.branch_data['scalename'])
        if self.my_debug:
          print('[DataComparison]: Yoda file name for order {} is: {}'.format(order,yodanames[order]))
        if os.path.isfile(yodanames[order]):
          os.remove(yodanames[order])

      # Loop over all observables and orders
      for obs in self.observables:
        for order in self.orders:

          # Define tab path and name
          tabbase = '{}.{}.{}'.format(self.process, order, obs)
          tabfile = tabbase + tabext
          logfile = tabbase + '_yodaout_' + self.branch_data['scalename'] + '.log'

          # Execute fnlo-tk-yodaout
          cmdlist = [ 'fnlo-tk-yodaout', tabfile, self.pdf, tabunc, order, ' _ ', self.branch_data['scalename'] ]
          if self.my_debug:
            print('[DataComparison]: DEBUG: Printing subprocess command list:\n{}'.format(cmdlist))
          with open(logfile, 'w') as log:
            code, out, error = interruptable_popen(cmdlist, stdout=log, stderr=log)
            if self.my_debug:
              print('[DataComparison]: DEBUG: Printing subprocess code: {}'.format(code))
              print('[DataComparison]: DEBUG: Printing subprocess output: {}'.format(out))
              print('[DataComparison]: DEBUG: Printing subprocess error: {}'.format(error))
            if (code != 0):
              raise Exception(error + 'fnlo-tk-yodaout returned non-zero exit status {}'.format(code))

          # Append yoda files separately for each order
          yinfiles = glob.glob('{}_*.yoda'.format(order))
          with open(yodanames[order],'a') as youtfile:
            for fname in yinfiles:
              with open(fname,'r') as yin:
                youtfile.write(yin.read())
              os.remove(fname)

      # Execute Rivet to produce data comparison plots for all grids (= histograms)
      cmdlist  = [ 'rivet-mkhtml', '--mc-errs', '-o{}'.format(self.branch_data['scalename']) ]
      for order in self.orders:
        if order == 'LO':
          cmdlist.append(yodanames[order]+':\'NNLOJET LO\':LineColor=green')
        elif order == 'NLO':
          cmdlist.append(yodanames[order]+':\'NLO\':LineColor=blue')
        elif order == 'NNLO':
          cmdlist.append(yodanames[order]+':\'NNLO\':LineColor=red:ErrorBands=1:ErrorBandColor=orange')
      if self.my_debug:
        print('[DataComparison]: DEBUG: Printing subprocess command list:\n{}'.format(cmdlist))
      code, out, error = interruptable_popen(cmdlist, stdout=PIPE , stderr=PIPE)
      if self.my_debug:
        print('[DataComparison]: DEBUG: Printing subprocess code: {}'.format(code))
        print('[DataComparison]: DEBUG: Printing subprocess output: {}'.format(out))
        print('[DataComparison]: DEBUG: Printing subprocess error: {}'.format(error))
      if (code != 0):
        raise Exception(error + 'rivet-mkhtml returned non-zero exit status {}'.format(code))

      # Create png plots, if missing for annoying security issues with ImageMagick convert used by Rivet
      pdir = os.getcwd()
      subdir = self.branch_data['scalename'] + "/" + self.rivetid
      os.chdir(subdir)
      if self.my_debug:
        print('[DataComparison]: Changed path to: {}'.format(os.getcwd()))
      pdffiles = glob.glob('*.pdf')
      for pdffile in pdffiles:
        outname = re.sub('.pdf', '', pdffile)
        outfile = outname + '.png'
        if not os.path.exists(outfile):
          cmdlist  = [ 'pdftoppm', '-png', '-singlefile', '-r', '150', pdffile, outname ]
          if self.my_debug:
            print('[DataComparison]: DEBUG: Printing subprocess command list:\n{}'.format(cmdlist))
          code, out, error = interruptable_popen(cmdlist, stdout=PIPE , stderr=PIPE)
          if self.my_debug:
            print('[DataComparison]: DEBUG: Printing subprocess code: {}'.format(code))
            print('[DataComparison]: DEBUG: Printing subprocess output: {}'.format(out))
            print('[DataComparison]: DEBUG: Printing subprocess error: {}'.format(error))
          if (code != 0):
            print('[DataComparison]: WARNING: Could not produce missing png plot!')
      os.chdir(pdir)
      if self.my_debug:
        print('[DataComparison]: Changed path back to: {}'.format(os.getcwd()))

      # Move produced folder with Rivet web pages and plots to destination in plots folder
      dstdir = opath + '/{}/{}/DataComparison'.format(self.plots_dir, self.name)
      if not os.path.isdir(dstdir):
        os.mkdir(dstdir)

      if self.my_debug:
        print('[DataComparison]: Current path: {}'.format(os.getcwd()))
        print('[DataComparison]: Destination path: {}'.format(dstdir))
      shutil.move(self.branch_data['scalename'], dstdir)

      # Change back to original path
      os.chdir(opath)
      if self.my_debug:
        print('[DataComparison]: Changed path back to: {}'.format(os.getcwd()))

    except:
      print('[DataComparison]: ERROR occured while running fnlo-tk-yodaout or rivet-mkhtml, aborted!')
      raise
