# -*- coding: utf-8 -*-

import law
import luigi
import glob
import os
import re
import shutil
from fnmatch import fnmatch
from subprocess import PIPE

from law.util import interruptable_popen

from Warmup import Warmup
from util import createRuncard

from analysis.framework import Task, HTCondorWorkflow

class FastWarm(Task, HTCondorWorkflow, law.LocalWorkflow):
  my_debug = luigi.BoolParameter(default=False)

  base_runcard = luigi.Parameter()
  steer_dir = luigi.Parameter()
  fastwarm_events = luigi.Parameter()
  channels = luigi.Parameter()
  process = luigi.Parameter()
  fastwarm_jobs = luigi.Parameter()
  starting_seeds = luigi.Parameter()

  jobok = False

  def create_branch_map(self):
    i = 0
    branchmap = {}
    for j, channel in enumerate(self.channels.split()):
      for k in xrange(int(self.fastwarm_jobs.split()[j])):
        branchmap[i] = {
          'index': j,
          'channel': channel,
          'events': self.fastwarm_events.split()[j],
          'seed': str(k + int(self.starting_seeds.split()[j]))
        }
        i += 1
    return branchmap

  def workflow_requires(self):
    return {
      'warmup': Warmup()
    }

  def requires(self):
    return {
      'warmup': Warmup(branch = self.branch_data['index'])
    }

  def output(self):
    return self.remote_target('{}.{}.{}.fastwarm.s{}.tar.gz'.format(self.process, self.branch_data['channel'], self.name, self.branch_data['seed']))

  def run(self):
    dirpath = 'tmpdir' + self.branch_data['seed']
    prevdir = os.getcwd()

    try:
      os.mkdir(dirpath)
      if not os.path.isfile(self.base_runcard):
        raise Exception('[FastWarm]: ERROR! Base runcard {} not found, aborted.'.format(self.base_runcard))
      shutil.move(self.base_runcard, dirpath+'/'+self.base_runcard)
      if not os.path.isdir(self.steer_dir):
        raise Exception('[FastWarm]: ERROR! Steerfile directory {} not found, aborted.'.format(self.steer_dir))
      os.chdir(self.steer_dir)
      strfiles = glob.glob('*.str')
      if not len(strfiles) > 0:
        raise Exception('[FastWarm]: ERROR! Steerfile directory {} empty, aborted.'.format(self.steer_dir))
      for file in strfiles:
        if self.my_debug:
          print('[FastWarm]: DEBUG: Moving steerfile {} to directory {}'.format(file,prevdir+'/'+dirpath))
        shutil.move(file, prevdir+'/'+dirpath+'/'+file)
      os.chdir(prevdir+'/'+dirpath)

      self.output().parent.touch()

      self.input()['warmup'].load('')

      outfile = os.path.basename(self.output().path)
      logfile = re.sub('.tar.gz$', '.log', outfile)
      runfile = re.sub('.tar.gz$', '.run', outfile)

      if self.my_debug:
        print('[FastWarm]: DEBUG: Out filename is: {}'.format(outfile))
        print('[FastWarm]: DEBUG: Log filename is: {}'.format(logfile))
        print('[FastWarm]: DEBUG: Run cardname is: {}'.format(runfile))

      with open(runfile, 'w') as outfile:
        runcard = createRuncard(open(self.base_runcard,'r').read(), {
          'channel': self.branch_data['channel'],
          'events': self.branch_data['events'],
          'seed': self.branch_data['seed'],
          'iterations': '1',
          'warmup': '0',
          'production': '.true.',
          'unit_phase': 'UNIT_PHASE'
        })
        outfile.write(runcard)

      if self.my_debug:
        print('[FastWarm]: DEBUG: Starting NNLOJET with runcard {}'.format(runfile))

      code, out, error = interruptable_popen(['NNLOJET', '-run', runfile], stdout=PIPE, stderr=PIPE)
      if self.my_debug:
        print('[FastWarm]: DEBUG: Printing subprocess code:\n{}\n'.format(code))
        print('[FastWarm]: DEBUG: Printing subprocess output:\n{}\n'.format(out))
        print('[FastWarm]: DEBUG: Printing subprocess error:\n{}\n'.format(error))
      with open(logfile, 'w') as outfile:
        outfile.write(out)
      if (code == 0):
        self.jobok = True
      else:
        raise Exception('[FastWarm]: ERROR! NNLOJET returned non-zero exit status {}'.format(code))

    finally:
      # Rename to individualise wrmfiles of each run for later merging
      for file in glob.glob('*.wrm'):
        wrmfile = re.sub('.wrm$', '.s' + self.branch_data['seed'] + '.wrm', file)
        if self.my_debug:
          print('[FastWarm]: DEBUG: Renaming {} to {}'.format(file, wrmfile))
        os.rename(file, wrmfile)

      # Collect results to be stored anyway in case of debug
      if self.jobok or self.my_debug:
        tarfilter = lambda n : n if (fnmatch(n.name, '*.wrm') or fnmatch(n.name, logfile) or fnmatch(n.name, runfile)) else None
        if self.jobok:
          self.output().dump(os.getcwd(), filter=tarfilter)
        else:
          self.output().dump(os.getcwd())
      # Clean up
      os.chdir(prevdir)
      shutil.rmtree(dirpath)
