# -*- coding: utf-8 -*-

import law
import luigi
import os
import re
import shutil
from law.util import interruptable_popen
from subprocess import PIPE

from CopyWarmup import CopyWarmup

from analysis.framework import Task

class PlotVegasGrids(Task, law.LocalWorkflow):
  my_debug = luigi.BoolParameter(default=False)
  force = luigi.BoolParameter(default=False)

  plots_dir = luigi.Parameter(default='plots')
  warm_dir  = luigi.Parameter(default='warmups')

  channels = luigi.Parameter()
  cutoff = luigi.Parameter()
  jobnameext = luigi.Parameter()
  kinetags = luigi.DictParameter()

  def create_branch_map(self):
    branchmap = {}
    for i, channel in enumerate(self.channels.split()):
      branchmap[i] = {
        'index': i,
        'channel': channel,
        'kinetag': self.kinetags[channel]
      }
    return branchmap

  def workflow_requires(self):
    if self.force:
      return {"copywarmup": CopyWarmup.req(self, acceptance=0)}
    else:
      return {"copywarmup": CopyWarmup.req(self)}

  def requires(self):
    return {
      'copywarmup': CopyWarmup(branch = self.branch_data['index'])
    }

  def output(self):
    target = '{}/{}/PlotVegasGrids/{}.{}-{}.{}.{}_{}'.format(self.plots_dir, self.name, self.process, self.branch_data['channel'], self.jobnameext, self.cutoff, self.branch_data['kinetag'], 'iterations.pdf')
    if self.my_debug:
      print('[PlotVegasGrids]: DEBUG: Local target is {}'.format(target))
    return law.LocalFileTarget(target)

  def run(self):
    self.output().parent.touch()
    gridpath = '{}/{}/{}'.format(self.warm_dir, 'warmup', self.name)
    gridfile = '{}.{}-{}.{}.{}_{}'.format(self.process, self.branch_data['channel'], self.jobnameext, self.cutoff, self.branch_data['kinetag'], 'iterations.txt')
    outpath = self.output().parent.path
    outfile = re.sub('.txt$', '.pdf', gridfile)
    if self.my_debug:
      print('[PlotVegasGrids]: DEBUG: The VEGAS grid path and filename are: {}/{}'.format(gridpath, gridfile))
      print('[PlotVegasGrids]: DEBUG: The VEGAS plot path and filename are: {}/{}'.format(outpath, outfile))

    try:
      code, out, error = interruptable_popen(['nnlojet-plotgrid.sh', gridpath+'/'+gridfile], stdout=PIPE, stderr=PIPE)
      if self.my_debug:
        print('[PlotVegasGrids]: DEBUG: Printing subprocess code:\n{}\n'.format(code))
        print('[PlotVegasGrids]: DEBUG: Printing subprocess output:\n{}\n'.format(out))
        print('[PlotVegasGrids]: DEBUG: Printing subprocess error:\n{}\n'.format(error))
      if (code != 0 or error):
        if 'STATS_columns' in error:
          print('[PlotVegasGrids]: ERROR: Check gnuplot version; might be too old (<v5) to use STATS_columns!')
        raise Exception('[PlotVegasGrids]: ERROR! {}; exitcode: {}'.format(error, code))

      shutil.move(gridpath+'/'+outfile, outpath+'/'+outfile)

    except:
      # Clean up failure
      if os.path.isfile(gridpath+'/'+outfile):
        os.remove(gridpath+'/'+outfile)
      if self.output().exists():
        self.output().remove()
      raise Exception('[PlotVegasGrids]: ERROR! Something went wrong. Cleaned output (if existing).')
