# -*- coding: utf-8 -*-

import law
import luigi
import os
import glob

from law.util import interruptable_popen
from subprocess import PIPE

from FastProd import FastProd
from FnloCppread import FnloCppread

from analysis.framework import Task

class SingleGridClosure(Task, law.LocalWorkflow):
  my_debug = luigi.BoolParameter(default=False)
  force = luigi.BoolParameter(default=False)

  merge_dir = luigi.Parameter(default='grids')
  plots_dir = luigi.Parameter(default='plots')

  channels = luigi.Parameter()
  observables = luigi.ListParameter()

  def create_branch_map(self):
    branchmap = {}
    i = 0
    for j, channel in enumerate(self.channels.split()):
      for observable in self.observables:
        branchmap[i] = {
          'index': j,
          'channel': channel,
          'observable': observable,
          'seed': FastProd().starting_seeds.split()[j]
        }
        i += 1
    return branchmap

  def workflow_requires(self):
    if self.force:
      return {"fnlocppread": FnloCppread.req(self, acceptance=0)}
    else:
      return {"fnlocppread": FnloCppread.req(self)}

  def requires(self):
    return {
      'fnlocppread': FnloCppread(branch = self.branch_data['index'])
    }

  def output(self):
    return law.LocalDirectoryTarget('{}/{}/SingleGridClosure/{}/{}'.format(self.plots_dir, self.name, self.branch_data['channel'], self.branch_data['observable']))

  def run(self):
    try:
      with self.output().temporary_path() as self.temp_output_path:
        os.mkdir(self.temp_output_path)

        tabglob = self.merge_dir + '/' + self.name + '/' + self.branch_data['channel'] + '/*.' + self.branch_data['observable'] + '.s' + self.branch_data['seed'] + '.tab.gz'
        # Only take the first one from all matches
        if self.my_debug:
          print('[SingleGridClosure]: DEBUG: Table glob: {}'.format(tabglob))
        tables = glob.glob(tabglob)
        if tables:
          tabfile = tables[0]
          if self.my_debug:
            print('[SingleGridClosure]: DEBUG: Table file: {}'.format(tabfile))
          outfile = '{}/{}.{}.{}'.format(self.temp_output_path, self.process, self.branch_data['channel'], self.branch_data['observable'])
          code, out, error = interruptable_popen(['fastnnlo_scaleclosure.py', tabfile, '-f', outfile], stdout=PIPE, stderr=PIPE)
          if self.my_debug:
            print('[SingleGridClosure]: DEBUG: Printing subprocess code:\n{}\n'.format(code))
            print('[SingleGridClosure]: DEBUG: Printing subprocess output:\n{}\n'.format(out))
            print('[SingleGridClosure]: DEBUG: Printing subprocess error:\n{}\n'.format(error))
        else:
          raise Exception('[SingleGridClosure]: ERROR! No matching file for table glob {}. Aborted.'.format(tabglob))

        if (code != 0):
          raise Exception('[SingleGridClosure]: ERROR! {} exitcode: {}'.format(error, code))
    except:
      os.rmdir(self.temp_output_path)
      raise
