# -*- coding: utf-8 -*-

import law
import luigi
import glob
import os
import re
import shutil
import tempfile
from fnmatch import fnmatch
from subprocess import PIPE

from law.util import interruptable_popen

from util import createRuncard

from analysis.framework import Task, HTCondorWorkflow

class Warmup(Task, HTCondorWorkflow, law.LocalWorkflow):
  my_debug = luigi.BoolParameter(default=False)

  base_runcard = luigi.Parameter()
  steer_dir = luigi.Parameter()
  channels = luigi.Parameter()
  warmup_events = luigi.Parameter()
  warmup_iterations = luigi.Parameter()
  starting_seed = luigi.Parameter()

  jobok = False

  def create_branch_map(self):
    branchmap = {}
    for i, channel in enumerate(self.channels.split()):
      branchmap[i] = {
        'channel': channel,
        'events': self.warmup_events.split()[i],
        'iterations': self.warmup_iterations.split()[i],
        'seed': str(i + int(self.starting_seed))
      }
    return branchmap

  def output(self):
    return self.remote_target('{}.{}.{}.warmup.tar.gz'.format(self.process, self.branch_data['channel'], self.name))

  def run(self):
    dirpath = 'tmpdir' + self.branch_data['seed']
    prevdir = os.getcwd()

    try:
      os.mkdir(dirpath)
      if not os.path.isfile(self.base_runcard):
        raise Exception('[Warmup]: ERROR! Base runcard {} not found, aborted.'.format(self.base_runcard))
      shutil.move(self.base_runcard, dirpath+'/'+self.base_runcard)
      os.chdir(dirpath)

      self.output().parent.touch()

      outfile = os.path.basename(self.output().path)
      logfile = re.sub('.tar.gz$', '.log', outfile)
      runfile = re.sub('.tar.gz$', '.run', outfile)

      if self.my_debug:
        print('[Warmup]: DEBUG: Out filename is: {}'.format(outfile))
        print('[Warmup]: DEBUG: Log filename is: {}'.format(logfile))
        print('[Warmup]: DEBUG: Run cardname is: {}'.format(runfile))

      with open(runfile, 'w') as outfile:
        runcard = createRuncard(open(self.base_runcard,'r').read(), {
          'channel': self.branch_data['channel'],
          'events': self.branch_data['events'],
          'seed': self.branch_data['seed'],
          'iterations': self.branch_data['iterations'],
          'warmup': '1',
          'production': '.false.',
          'unit_phase': ''
        })
        outfile.write(runcard)

      # KR: With hyperthreading it might be necessary to request more cpus on a node than one wants to use for the calculation!
      #     --> Differentiate between request and use CPUs!
      #      os.environ['OMP_NUM_THREADS'] = self.htcondor_request_cpus
      os.environ['OMP_NUM_THREADS'] = self.htcondor_use_cpus

      if self.my_debug:
        print('[Warmup]: DEBUG: Starting NNLOJET with runcard {}'.format(runfile))

      code, out, error = interruptable_popen(['NNLOJET', '-run', runfile], stdout=PIPE, stderr=PIPE)
      if self.my_debug:
        print('[Warmup]: DEBUG: Printing subprocess code:\n{}\n'.format(code))
        print('[Warmup]: DEBUG: Printing subprocess output:\n{}\n'.format(out))
        print('[Warmup]: DEBUG: Printing subprocess error:\n{}\n'.format(error))
      # ATTENTION: Frequently, NNLOJET does NOT report errors as such (code > 0)!
      # ==> Catch NNLOJET seed error with code 0
      errstr1 = 'Error, index request'
      errstr2 = 'Error: define a seed'
      if errstr1 in out or errstr2 in out:
        code = 1
        error = out
      with open(logfile, 'w') as outfile:
        outfile.write(out)
      if (code == 0):
        self.jobok = True
      else:
        raise Exception('[Warmup]: ERROR! NNLOJET returned non-zero exit status {}'.format(code))

    finally:
      # Collect results to be stored anyway in case of debug
      if self.jobok or self.my_debug:
        tarfilter = lambda n : n if fnmatch(n.name, '*{}*'.format(self.branch_data['channel'])) else None
        if self.jobok:
          self.output().dump(os.getcwd(), filter=tarfilter)
        else:
          self.output().dump(os.getcwd())
      # Clean up
      os.chdir(prevdir)
      shutil.rmtree(dirpath)
